open Cmdliner
open Logarion
module C = Archive.Configuration
module Lpath = Logarion.Lpath

let conf () =
  let module Config = Confix.Config.Make (Confix.ConfixToml) in
  let archive_res = Config.config "config.toml" |> Config.to_record C.of_config in
  match archive_res with
  | Ok config -> config
  | Error str -> prerr_endline str; exit 1

let init =
  let f force =
    let repo = Lpath.string_of_repo @@ (conf ()).C.repository in
    let make_dir description d =
      let open Unix in
      try
        mkdir d 0o700;
        print_endline ("created " ^ description ^ ": " ^ d)
      with Unix_error (EEXIST, "mkdir", _) ->
        prerr_endline (description ^ " already exists: " ^ d);
        if Array.length (Sys.readdir repo) > 0 then prerr_endline ("warning: " ^ d ^ " is not empty")
    in
    make_dir "repository" repo
  in
  let force =
    Arg.(value & flag & info ["f"; "force"] ~doc:"Initialise repository even if directory is non empty")
  in
  Term.(const f $ force),
  Term.info
    "init" ~doc:"initialise a logarion repository in present directory"
    ~man:[ `S "DESCRIPTION"; `P "Create a repository in current directory" ]

let create =
  let title =
    Arg.(value & pos 0 string "" & info [] ~docv:"TITLE" ~doc:"Title for new article")
  in
  let f title =
    let repo = (conf ()).C.repository in
    let t = match title with "" -> "Draft" | _ -> title in
    let note = Note.({ (blank ()) with meta = { (Meta.blank ()) with Meta.title = t }}) in
    File.Lwt.with_note (File.store repo) note
    |> Lwt_main.run
    |> ignore
  in
  Term.(const f $ title),
  Term.info "create"
            ~doc:"create a new article"
            ~man:[ `S "DESCRIPTION";  `P "Create a new article, with title 'Draft' when none provided"]

let default_cmd =
  Term.(ret (const (`Help (`Pager, None)))),
  Term.info "logarion" ~version:"0.1.0" ~doc:"an article collection & publishing system"
            ~man:[ `S "BUGS"; `P "Submit bugs https://gitlab.com/orbifx/logarion/issues/new."; ]

let cmds = [ init; create ]

let () = match Term.eval_choice default_cmd cmds with
  | `Error _ -> exit 1 | _ -> exit 0
